/*-
 * Copyright (c) 2007 Matus Harvan
 * All rights reserved
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 *       copyright notice, this list of conditions and the following
 *       disclaimer in the documentation and/or other materials provided
 *       with the distribution.
 *     * The name of the author may not be used to endorse or promote
 *       products derived from this software without specific prior
 *       written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/ioctl.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <unistd.h>
#include <err.h>
#include <net/if_tun.h>

/* 
 * Allocate TUN device, returns opened fd. 
 * Stores dev name in the first arg(must be large enough).
 */  
int tun_open(char *dev)
{
	char tunname[14];
	int i, fd = -1;
	
	if( *dev ){
		sprintf(tunname, "/dev/%s", dev);
		fd = open(tunname, O_RDWR);
	} else {
		for(i=0; i < 255; i++){
			sprintf(tunname, "/dev/tun%d", i);
			/* Open device */
			if( (fd=open(tunname, O_RDWR)) > 0 ){
				sprintf(dev, "tun%d", i);
				break;
			}
		}
	}
	if( fd > -1 ){
		i=0;
		/* Disable extended modes */
		ioctl(fd, TUNSLMODE, &i);	
		ioctl(fd, TUNSIFHEAD, &i);
	}	
	return fd;
}

int tun_close(int fd, char *dev)
{
	return close(fd);
}

int tun_write(int fd, char *buf, int len)
{
	return write(fd, buf, len);
}

int tun_read(int fd, char *buf, int len)
{
	return read(fd, buf, len);
}

int 
tun_setmtu(char *dev, const int mtu)
{
        char cmdline[512];
	
        if (mtu > 200 && mtu < 1500) {
                snprintf(cmdline, sizeof(cmdline), "/sbin/ifconfig %s mtu %d",
		    dev, mtu);
                return system(cmdline);
        } else {
                warnx("MTU out of range: %d\n", mtu);
		return 1;
        }
}
