#include <arpa/inet.h>
#include <errno.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <sysexits.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <time.h>
#include <unistd.h>

#include <netinet/in.h>
#include <netinet/udp.h>

#define SOCKET_TIMEOUT 10
#define BUFLEN 1500


static void
dump(char *data, int len)
{
    int i;
    printf("Dumping %d bytes (as hex)...\n", len);
    for (i = 0; i < len; i++)
	printf("%02hhx", *(data+i));
    printf("\n");
}

int
main()
{
    int fd;
    int new_fd;
    char *msg = "Welcome to catchalld\r\n";
    int n, nwrite, nread;
    int soval = 1;
    int count = 5;

    struct sockaddr_storage from;
    socklen_t fromlen = sizeof(from);
    char host[NI_MAXHOST];
    char serv[NI_MAXSERV];

    char buf[BUFLEN];
    char *bufp;
    int buflen;

    struct ip *iphdr;
    struct udphdr *uhdr;    
    char *payload;    
    int payload_len;
    struct sockaddr_in sa1;
    struct sockaddr_in sa2;

    memset(&sa1, 0, sizeof(sa1));
    sa1.sin_addr.s_addr = INADDR_ANY;

    fd = socket(AF_INET, SOCK_RAW, IPPROTO_UDP);
    if (fd < 0)
	err(EX_OSERR, "Failed to create a raw UDP socket");
    
    while (count > 0) {
	/* wait for traffic */
	nread = recvfrom(fd, buf, BUFLEN, 0,
			 (struct sockaddr *) &from, &fromlen);
	printf("recvfrom() returned %d\n", nread);
	if (nread < 0) {
	    warn("recvfrom() returned %d", nread);
	    continue;
	}

	if (getnameinfo((struct sockaddr *) &from, fromlen,
			host, sizeof(host), serv, sizeof(serv),
			NI_NUMERICHOST | NI_DGRAM) ) {
	    fprintf(stderr, "getnameinfo failed: %s\n",
		    gai_strerror(errno));
	} else {
	    fprintf(stderr, "received traffic from client %s:%s\n",
		    host, serv);
	}

	fprintf(stderr, "traffic on UDP socket \n");
	bufp = buf;
	buflen = nread;
	dump(bufp, buflen);

	/* parse the UDP header */
	if (nread >= sizeof(struct udphdr)) {
		/* IP header */
		iphdr = (struct ip *)bufp;
		bufp += 20;
		buflen -= 20;
		printf("IP hdr: ");
		dump((void*)iphdr, 20);

		/* UDP header */
		uhdr = (struct udphdr *)bufp;
		bufp += sizeof(*uhdr);
		buflen -= sizeof(*uhdr);
		printf("UDP hdr: ");
		dump((void*)uhdr, sizeof(*uhdr));
		printf("dport: %d, sport: %d\n", 
		       ntohs(uhdr->uh_dport), ntohs(uhdr->uh_sport));
		sa1.sin_port = uhdr->uh_dport;
		memcpy(&sa2, &from, sizeof(sa2));
		sa2.sin_port = uhdr->uh_sport;

		/* payload */
		payload = bufp;
		payload_len = buflen;
		printf("Payload: ");
		dump(payload, payload_len);
		int i;
		for (i=0; i<payload_len; i++)
			printf("%c", *(payload+i));
		printf("\n");
	} else {
	    warnx("received packet too short to contain a UDP header\n");
	}

	n = 0;
	nwrite = 0;
	do {
	    //new_fd = udp_open("1234");

	    new_fd = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
	    if (new_fd == -1)
		err(EX_UNAVAILABLE, "socket() failed");
	    else
		printf("socket() returned %d\n", new_fd);

	    int nbind =
		bind(new_fd, (struct sockaddr *)&sa1, sizeof(struct sockaddr));
	    if (nbind != 0)
		err(EX_NOHOST, "bind failed");
	    else
		printf("bind returned %d\n", nbind);


	    nwrite = sendto(new_fd, msg, strlen(msg), 0,
			    (struct sockaddr *) &sa2, sizeof(sa2));
	    if (nwrite < 0)
		warn("sendto() returned %d", nwrite);
	    else if (nwrite == 0)
		warnx("sendto() returned %d", nwrite);
	    else
		printf("sendto() returned %d\n", nwrite);

	    n += nwrite;
	} while (n < strlen(msg) && nwrite > 0);
	count--;
	close(new_fd);
    }
    close(fd);
    return 0;
}
